#ifndef GLWIDGET_H
#define GLWIDGET_H

#include <QtOpenGL/QGLWidget>
#include <QTimer>
#include <GL/gl.h>
#include "subwindowinterface.h"

class CellSpace;

class GLwidget : public QGLWidget, public SubWindowInterface
{
    Q_OBJECT
    protected:
    void initializeGL();
    void paintGL();
    void resizeGL(int w, int h);
    public:
    GLwidget(const QGLFormat &format, QWidget *parent = 0, CellSpace *space = NULL, QGLWidget *share = 0);
    ~GLwidget();
    void loadTexture(int w, int h, unsigned char *texture_data);
    QSize sizeHint()const{ return QSize(300, 300); }
    void mouseMoveEvent(QMouseEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void wheelEvent(QWheelEvent *event);
    void startSimulation();
    void stopSimulation();
    static int getUntitledNumber();
    CellSpace* getSpace(){ return space; }
    //SubWindowInterface metody
    void setSubWindowTitle();
    int load(const QString &file);
    QString save(const QString &dir);
    QString getFileName();
    public slots:
    void runSimulation();
    void saveImage(QString &file);
    void setName(QString &name){ spaceName = name; }
    private:
    GLuint tex;
    static bool initialized;
    static int number;
    QString spaceName;
    float zoom;
    float pos_x,pos_y;
    int w,h;
    int tex_w,tex_h;
    int mouse_x,mouse_y;
    QTimer timer;
    CellSpace *space;
};

#endif // GLWIDGET_H
